@@ -1,5 +1,8 @@
 import os, sys
 
+if (sys.version_info.major == 2) and (sys.version_info.minor < 7):
+    raise RuntimeError("Python 2.7 or 3.3 are required to run rpy2")
+
 try:
     R_HOME = (os.environ["R_HOME"], )
 except KeyError:
@@ -8,13 +11,8 @@ except KeyError:
     tmp.close()
     del(tmp)
 
-# MSWindows-specific
-_win_bindirs = {'win32': 'i386',
-                'win-amd64': 0, #FIXME: where are the 64 bit-binaries located ?
-                'win-ia64': 0}
-
 if len(R_HOME) == 0:
-    if sys.platform in _win_bindirs.keys():
+    if sys.platform == 'win32':
         try:
             import win32api
             import win32con
@@ -53,7 +51,16 @@ os.environ['R_HOME'] = R_HOME
 
 # MSWindows-specific code
 _win_ok = False
-if sys.platform in _win_bindirs.keys():
+if sys.platform == 'win32':
+    import platform
+    architecture = platform.architecture()[0]
+    if architecture == '32bit':
+        _win_bindir = 'i386'
+    elif architecture == '64bit':
+        _win_bindir = 'x64'
+    else:
+        raise ValueError("Unknown architecture %s" %architecture)
+
     import win32api
     os.environ['PATH'] += ';' + os.path.join(R_HOME, 'bin')
     os.environ['PATH'] += ';' + os.path.join(R_HOME, 'modules')
@@ -61,15 +68,14 @@ if sys.platform in _win_bindirs.keys():
 
     # Load the R dll using the explicit path
     R_DLL_DIRS = ('bin', 'lib')
-    # Try dirs in R_DLL_DIRS
+    # Try dirs from R_DLL_DIRS
     for r_dir in R_DLL_DIRS:
-        Rlib = os.path.join(R_HOME, r_dir, _win_bindirs[sys.platform], 'R.dll')
+        Rlib = os.path.join(R_HOME, r_dir, _win_bindir, 'R.dll')
         if not os.path.exists(Rlib):
             continue
-        win32api.LoadLibrary(Rlib)
+        win32api.LoadLibrary( Rlib )
         _win_ok = True
         break
-
     # Otherwise fail out!
     if not _win_ok:
         raise RuntimeError("Unable to locate R.dll within %s" % R_HOME)
@@ -88,8 +94,15 @@ from rpy2.rinterface._rinterface import *
 
 
 # wrapper in case someone changes sys.stdout:
-def consolePrint(x):
-    sys.stdout.write(x)
+if sys.version_info.major == 3:
+    # Print became a regular function in Python 3, making
+    # the workaround (mostly) unnecessary (python2to3 still needs it
+    # wrapped in a function
+    def consolePrint(x):
+        print(x)
+else:
+    def consolePrint(x):
+        sys.stdout.write(x)
 
 set_writeconsole(consolePrint)
 
@@ -144,5 +157,3 @@ def rternalize(function):
 #     return True
 
 # setCleanUp(cleanUp)
-
-
