@@ -12,7 +12,7 @@
  * for the specific language governing rights and limitations under the
  * License.
  * 
- * Copyright (C) 2008-2011 Laurent Gautier
+ * Copyright (C) 2008-2012 Laurent Gautier
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -203,9 +203,9 @@ static PyTypeObject NAInteger_Type = {
         0,                      /*tp_methods*/
         0,                      /*tp_members*/
         0,                      /*tp_getset*/
-#ifdef Win32
+#if defined(Win32) || defined(Win64)
 	NULL,
-#else
+#else	
         &PyLong_Type,           /*tp_base*/
 #endif
         0,                      /*tp_dict*/
@@ -401,7 +401,7 @@ static PyTypeObject NALogical_Type = {
         0, //NAInteger_methods,           /*tp_methods*/
         0,                      /*tp_members*/
         0,                      /*tp_getset*/
-#ifdef Win32
+#if defined(Win32) || defined(Win64)
 	NULL,
 #else
         &PyLong_Type,             /*tp_base*/
@@ -587,11 +587,11 @@ static PyTypeObject NAReal_Type = {
         0, //NAInteger_methods,           /*tp_methods*/
         0,                      /*tp_members*/
         0,                      /*tp_getset*/
-#ifdef Win32
+#if defined(Win32) || defined(Win64)
 	NULL,
 #else
         &PyFloat_Type,             /*tp_base*/
-#endif
+#endif	
         0,                      /*tp_dict*/
         0,                      /*tp_descr_get*/
         0,                      /*tp_descr_set*/
@@ -689,7 +689,7 @@ static PyTypeObject NACharacter_Type = {
         0, //NAInteger_methods,           /*tp_methods*/
         0,                      /*tp_members*/
         0,                      /*tp_getset*/
-#ifdef Win32
+#if defined(Win32) || defined(Win64)
 	NULL,
 #elif (PY_VERSION_HEX < 0x03010000)
         &PyString_Type,             /*tp_base*/
@@ -856,7 +856,7 @@ static PyTypeObject NAComplex_Type = {
         0, //NAInteger_methods,           /*tp_methods*/
         0,                      /*tp_members*/
         0,                      /*tp_getset*/
-#ifdef Win32
+#if defined(Win32) || defined(Win64)
 	NULL,
 #else
         &PyComplex_Type,             /*tp_base*/
