@@ -12,7 +12,7 @@
  * for the specific language governing rights and limitations under the
  * License.
  * 
- * Copyright (C) 2008-2011 Laurent Gautier
+ * Copyright (C) 2008-2012 Laurent Gautier
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -30,39 +30,11 @@
 
 #include <Rdefines.h>
 
-
-SEXP rpy_findFun(SEXP symbol, SEXP rho)
-{
-    SEXP vl;
-    while (rho != R_EmptyEnv) {
-        /* This is not really right.  Any variable can mask a function */
-        vl = findVarInFrame3(rho, symbol, TRUE);
-
-        if (vl != R_UnboundValue) {
-            if (TYPEOF(vl) == PROMSXP) {
-                PROTECT(vl);
-                vl = eval(vl, rho);
-                UNPROTECT(1);
-            }
-            if (TYPEOF(vl) == CLOSXP || TYPEOF(vl) == BUILTINSXP ||
-                TYPEOF(vl) == SPECIALSXP)
-               return (vl);
-
-            if (vl == R_MissingArg) {
-              printf("R_MissingArg in rpy_FindFun.\n");
-              return R_UnboundValue;
-            }
-        }
-        rho = ENCLOS(rho);
-    }
-    return R_UnboundValue;
-}
-
 SEXP rpy_serialize(SEXP object, SEXP rho)
 {
   SEXP c_R, call_R, res, fun_R;
 
-  PROTECT(fun_R = rpy_findFun(install("serialize"), rho));
+  PROTECT(fun_R = PyRinterface_FindFun(install("serialize"), rho));
   if(!isEnvironment(rho)) error("'rho' should be an environment");
   /* obscure incatation to summon R */
   PROTECT(c_R = call_R = allocList(3));
@@ -85,7 +57,7 @@ SEXP rpy_serialize(SEXP object, SEXP rho)
 SEXP rpy_unserialize(SEXP connection, SEXP rho)
 {
   SEXP c_R, call_R, res, fun_R;
-  PROTECT(fun_R = rpy_findFun(install("unserialize"), rho));
+  PROTECT(fun_R = PyRinterface_FindFun(install("unserialize"), rho));
   if(!isEnvironment(rho)) error("'rho' should be an environment");
   /* obscure incatation to summon R */
   PROTECT(c_R = call_R = allocList(2));
@@ -126,10 +98,16 @@ SEXP rpy_list_attr(SEXP sexp)
 
 SEXP rpy_remove(SEXP symbol, SEXP env, SEXP rho)
 {
-  SEXP c_R, call_R, res, fun_R;
-
-  PROTECT(fun_R = rpy_findFun(install("rm"), rho));
+  SEXP c_R, call_R, res;
 
+  static SEXP fun_R = NULL;
+  if (fun_R == NULL) {
+    PROTECT(fun_R = PyRinterface_FindFun(install("rm"), rho));
+    /* FIXME: or use the rpy2 object tracking layer ?*/
+    R_PreserveObject(fun_R);
+    UNPROTECT(1);
+  }
+  
   if(!isEnvironment(rho)) error("'rho' should be an environment");
   /* incantation to summon R */
   PROTECT(c_R = call_R = allocList(2+1));
@@ -151,6 +129,7 @@ SEXP rpy_remove(SEXP symbol, SEXP env, SEXP rho)
   int error = 0;
   PROTECT(res = R_tryEval(call_R, rho, &error));
 
-  UNPROTECT(3);
+  UNPROTECT(2);
   return res;
 }
+
