@@ -45,12 +45,33 @@ class PackagesTestCase(unittest.TestCase):
                           robjects.packages.Package,
                           env, "dummy_package")
 
+class SignatureTranslatedAnonymousPackagesTestCase(unittest.TestCase):
+    def testNew(self):
+        string = """
+   square <- function(x) {
+       return(x^2)
+   }
+
+   cube <- function(x) {
+       return(x^3)
+   }
+   """
+        powerpack = packages.SignatureTranslatedAnonymousPackage(string, "powerpack")
+        self.assertTrue(hasattr(powerpack, 'square'))
+        self.assertTrue(hasattr(powerpack, 'cube'))
+
 
 class ImportrTestCase(unittest.TestCase):
     def testImportStats(self):
         stats = robjects.packages.importr('stats')
         self.assertTrue(isinstance(stats, robjects.packages.Package))
-
+        
+    def testImportDatasets(self):
+        datasets = robjects.packages.importr('datasets')
+        self.assertTrue(isinstance(datasets, robjects.packages.Package))
+        self.assertTrue(isinstance(datasets.data, 
+                                   robjects.packages.PackageData))
+        
 class WherefromTestCase(unittest.TestCase):
     def testWherefrom(self):
         stats = robjects.packages.importr('stats')
@@ -61,6 +82,8 @@ class WherefromTestCase(unittest.TestCase):
 def suite():
     suite = unittest.TestLoader().loadTestsFromTestCase(PackagesTestCase)
     suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ImportrTestCase))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(WherefromTestCase))
+    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(SignatureTranslatedAnonymousPackagesTestCase))
     return suite
 
 if __name__ == '__main__':
