@@ -5,11 +5,11 @@ import conversion
 
 import rpy2.rlike.container as rlc
 
-import sys, copy, os, itertools
+import sys, copy, os, itertools, math
 import time
 from time import struct_time, mktime
 
-from rpy2.rinterface import Sexp, SexpVector, StrSexpVector, IntSexpVector, BoolSexpVector, ComplexSexpVector, FloatSexpVector, R_NilValue, NA_Real, NA_Integer, NA_Character, NA_Logical, NULL
+from rpy2.rinterface import Sexp, SexpVector, ListSexpVector, StrSexpVector, IntSexpVector, BoolSexpVector, ComplexSexpVector, FloatSexpVector, R_NilValue, NA_Real, NA_Integer, NA_Character, NA_Logical, NULL
 
 globalenv_ri = rinterface.globalenv
 baseenv_ri = rinterface.baseenv
@@ -181,6 +181,10 @@ class VectorOperationsDelegator(object):
         res = globalenv_ri.get("-")(self._parent)
         return res
 
+    def __contains__(self, what):
+        res = globalenv_ri.get("%in%")(self._parent, what)
+        return res
+
 
 class Vector(RObjectMixin, SexpVector):
     """ Vector(seq) -> Vector.
@@ -282,13 +286,13 @@ class Vector(RObjectMixin, SexpVector):
                 if len(x) < max_width:
                     res = x
                 else:
-                    res = "%s..." %x[ : (max_width - 3)]
+                    res = "%s..." % (str(x[ : (max_width - 3)]))
             return res
 
         l = len(self)
         if l < 7:
             s = '[' + \
-                ', '.join((p_str(x, max_width = 52 / l) for x in self[ : 8])) +\
+                ', '.join((p_str(x, max_width = math.floor(52 / l)) for x in self[ : 8])) +\
                 ']'
         else:
             s = '[' + \
@@ -349,7 +353,8 @@ class IntVector(Vector, IntSexpVector):
         super(IntVector, self).__init__(obj)
 
     def tabulate(self, nbins = None):
-        """ Count the number of times integer values are found """
+        """ Like the R function tabulate,
+        count the number of times integer values are found """
         if nbins is None:
             nbins = max(1, max(self))
         res = self._tabulate(self)
@@ -391,7 +396,7 @@ class FloatVector(Vector, FloatSexpVector):
     The parameter 'seq' can be an instance inheriting from
     rinterface.SexpVector, or an arbitrary Python sequence.
     In the later case, all elements in the sequence should be either
-    float, or have an float() representation.
+    float, or have a float() representation.
 
     """
     def __init__(self, obj):
@@ -464,7 +469,7 @@ class FactorVector(IntVector):
         for x in self:
             yield levels[x-1]
 
-class ListVector(Vector):
+class ListVector(Vector, ListSexpVector):
     """ R list (vector of arbitray elements)
 
     ListVector(iteritemable) -> ListVector.
@@ -653,15 +658,15 @@ class Array(Vector):
 
     def __dimnames_get(self):
         """ Return a list of name vectors
-        (like the R function 'dimnames' does it)."""
+        (like the R function 'dimnames' does)."""
 
         res = self._dimnames_get(self)
         res = conversion.ri2py(res)
         return res
 
     def __dimnames_set(self, value):
-        """ Return a list of name vectors
-        (like the R function 'dimnames' does it)."""
+        """ Set list of name vectors
+        (like the R function 'dimnames' does)."""
 
         value = conversion.ri2py(value)
         res = self._dimnames_set(self, value)        
